//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Tools::InstanceCounter
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   16.12.2001
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2005-03-08 21:46:27 +0100 (Di, 08 Mär 2005) $
 *   $Revision: 1.3 $
 */
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <map>

//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>
#include <Ax/Tools/Globals.h>
#include <Ax/Tools/InstanceCounter.h>

//----- local -----------------------------------------------------------------

//=============================================================================
//     using namespaces
//=============================================================================
namespace Ax
{
namespace Tools
{

//=============================================================================
//     preprocessor
//=============================================================================
#ifndef HAVE_INSTANCE_COUNTER
#define HAVE_INSTANCE_COUNTER 1
#endif

//=============================================================================
//     globals
//=============================================================================
#if HAVE_INSTANCE_COUNTER
typedef std::map<std::string, long> TInstanceCltn;
typedef TInstanceCltn::iterator     TInstanceCltnIter;

static TInstanceCltn InstanceCltn;
#endif

//=============================================================================
//     class InstanceCounter
//=============================================================================

//---------------------------------------------------------------------
//     InstanceCounter::InstanceCounter()
//---------------------------------------------------------------------
InstanceCounter::InstanceCounter(const std::string &theClassName)
               : myClassName(theClassName)
{
#if HAVE_INSTANCE_COUNTER
  wsdebug(("> InstanceCounter::InstanceCounter() class: '%s'", myClassName.c_str()));

  incRef(myClassName);

  wsdebug(("< InstanceCounter::InstanceCounter() class: '%s'", myClassName.c_str()));
#endif
} // InstanceCounter::InstanceCounter()

//---------------------------------------------------------------------
//     InstanceCounter::InstanceCounter()
//---------------------------------------------------------------------
InstanceCounter::InstanceCounter(const InstanceCounter &that)
               : myClassName(that.myClassName)
{
#if HAVE_INSTANCE_COUNTER
  wsdebug(("> InstanceCounter::InstanceCounter(const InstanceCounter &) class: '%s'", myClassName.c_str()));

  incRef(myClassName);

  wsdebug(("< InstanceCounter::InstanceCounter(const InstanceCounter &) class: '%s'", myClassName.c_str()));
#endif
} // InstanceCounter::InstanceCounter()

//---------------------------------------------------------------------
//     InstanceCounter::~InstanceCounter()
//---------------------------------------------------------------------
InstanceCounter::~InstanceCounter()
{
#if HAVE_INSTANCE_COUNTER
  wsdebug(("> InstanceCounter::~InstanceCounter() class: '%s'", myClassName.c_str()));

  decRef(myClassName);

  wsdebug(("< InstanceCounter::~InstanceCounter() class: '%s'", myClassName.c_str()));
#endif
} // InstanceCounter::~InstanceCounter()

//---------------------------------------------------------------------
//     InstanceCounter::operator=()
//---------------------------------------------------------------------
InstanceCounter &InstanceCounter::operator=(const InstanceCounter &that)
{
  AX_UNUSED_ARG(that);
#if HAVE_INSTANCE_COUNTER
  wsdebug(("> InstanceCounter::operator=() class: '%s'", myClassName.c_str()));

  if (this != &that && myClassName != that.myClassName)
  {
    decRef(myClassName);
    myClassName = that.myClassName;
    incRef(myClassName);
  } // if

  wsdebug(("< InstanceCounter::operator=() class: '%s'", myClassName.c_str()));
#endif
  return *this;
} //   InstanceCounter::operator=()

//---------------------------------------------------------------------
//     InstanceCounter::dump()
//---------------------------------------------------------------------
void InstanceCounter::dump(const std::string &theMsg)
{
  AX_UNUSED_ARG(theMsg);
#if HAVE_INSTANCE_COUNTER
  wswarn((">>> ---------------------------------------------------------"));
  if (InstanceCltn.size() > 0)
  {
    wswarn(("InstanceCounter: Following classes/instances are registered: "));
    if (theMsg.length() > 0)
    {
      wswarn(("at: %s", theMsg.c_str()));
    } // if

    for (TInstanceCltnIter anIter  = InstanceCltn.begin();
                           anIter != InstanceCltn.end();
                         ++anIter)
    {
      wswarn(("%8ld: %s", anIter->second, anIter->first.c_str()));
    } // for
  }
  else
  {
    wswarn(("InstanceCounter: No classes/instances are registered!"));
    if (theMsg.length() > 0)
    {
      wswarn(("at: %s", theMsg.c_str()));
    } // if
  } // if
  wswarn(("<<< ---------------------------------------------------------"));
#endif
} // InstanceCounter::dump()

//---------------------------------------------------------------------
//     InstanceCounter::incRef()
//---------------------------------------------------------------------
void InstanceCounter::incRef(const std::string &theClassName)
{
  AX_UNUSED_ARG(theClassName);
#if HAVE_INSTANCE_COUNTER
  TInstanceCltnIter aPos = InstanceCltn.find(theClassName);
  if (aPos == InstanceCltn.end())
  {
    InstanceCltn[theClassName] = 1;
  }
  else
  {
    aPos->second++;
  } // if
#endif
} // InstanceCounter::incRef()

//---------------------------------------------------------------------
//     InstanceCounter::decRef()
//---------------------------------------------------------------------
void InstanceCounter::decRef(const std::string &theClassName)
{
  AX_UNUSED_ARG(theClassName);
#if HAVE_INSTANCE_COUNTER
  TInstanceCltnIter aPos = InstanceCltn.find(theClassName);
  if (aPos != InstanceCltn.end())
  {
    aPos->second--;
    if (aPos->second < 0)
    {
      wswarn(("InstanceCounter:: more instances of class '%s' destroyed than created!"
            , theClassName.c_str()
            ));
    } // if
  }
  else
  {
    wswarn(("InstanceCounter:: class '%s' not counted!", theClassName.c_str()));
  } // if
#endif
} // InstanceCounter::decRef()

} // namespace Tools
} // namespace Ax
